@@ -23,11 +23,11 @@ Artifactory::Client - Perl client for Artifactory REST API
 
 =head1 VERSION
 
-Version 1.0.0
+Version 1.1.0
 
 =cut
 
-our $VERSION = 'v1.0.0';
+our $VERSION = 'v1.1.0';
 
 =head1 SYNOPSIS
 
@@ -294,6 +294,24 @@ sub build_promotion {
     );
 }
 
+=head2 promote_docker_image( targetRepo => "target_repo", dockerRepository => "dockerRepository", tag => "tag", copy => 'false' )
+
+Promotes a Docker image from one repository to another
+
+=cut
+
+sub promote_docker_image {
+    my ( $self, %data ) = @_;
+
+    my $repo = $self->repository();
+    my $url  = $self->_api_url() . "/docker/$repo/v2/promote";
+    return $self->post(
+        $url,
+        "Content-Type" => 'application/json',
+        Content        => $self->_json->encode( \%data )
+    );
+}
+
 =head2 delete_builds( name => $build_name, buildnumbers => [ buildnumbers ], artifacts => 0,1, deleteall => 0,1 )
 
 Removes builds stored in Artifactory. Useful for cleaning up old build info data
@@ -888,6 +906,23 @@ sub delete_local_multi_push_replication {
     return $self->delete($call_url);
 }
 
+=head2 enable_or_disable_multiple_replications( 'enable|disable', include => [ ], exclude => [ ] )
+
+Enables/disables multiple replication tasks by repository or Artifactory server based in include and exclude patterns.
+
+=cut
+
+sub enable_or_disable_multiple_replications {
+    my ( $self, $flag, %info ) = @_;
+    my $repo = $self->repository();
+    my $url  = $self->_api_url() . "/replications/$flag";
+    return $self->post(
+        $url,
+        "Content-Type" => 'application/json',
+        Content        => $self->_json->encode( \%info )
+    );
+}
+
 =head2 artifact_sync_download( $path, content => 'progress', mark => 1000 )
 
 Downloads an artifact with or without returning the actual content to the client. When tracking the progress marks are
@@ -924,6 +959,69 @@ sub file_list {
     return $self->get($url);
 }
 
+=head2 get_background_tasks
+
+Retrieves list of background tasks currently scheduled or running in Artifactory. In HA, the nodeId is added to each
+task. Task can be in one of few states: scheduled, running, stopped, canceled. Running task also shows the task start
+time.
+
+=cut
+
+sub get_background_tasks {
+    my $self = shift;
+    my $url  = $self->_api_url() . "/tasks";
+    return $self->get($url);
+}
+
+=head2 empty_trash_can
+
+Empties the trash can permanently deleting all its current contents.
+
+=cut
+
+sub empty_trash_can {
+    my $self = shift;
+    my $url  = $self->_api_url() . "/trash/empty";
+    return $self->post($url);
+}
+
+=head2 delete_item_from_trash_can($path)
+
+Permanently deletes an item from the trash can.
+
+=cut
+
+sub delete_item_from_trash_can {
+    my ( $self, $path ) = @_;
+    my $url = $self->_api_url() . "/trash/$path";
+    return $self->delete($url);
+}
+
+=head2 restore_item_from_trash_can( $from, $to )
+
+Restore an item from the trash can.
+
+=cut
+
+sub restore_item_from_trash_can {
+    my ( $self, $from, $to ) = @_;
+    my $url = $self->_api_url() . "/trash/restore/$from?to=$to";
+    return $self->post($url);
+}
+
+=head2 optimize_system_storage
+
+Raises a flag to invoke balancing between redundant storage units of a sharded filestore following the next garbage
+collection.
+
+=cut
+
+sub optimize_system_storage {
+    my $self = shift;
+    my $url  = $self->_api_url() . "/system/storage/optimize";
+    return $self->post($url);
+}
+
 =head1 SEARCHES
 
 =cut
@@ -1142,6 +1240,19 @@ sub build_artifacts_search {
     );
 }
 
+=head2 list_docker_repositories
+
+Lists all Docker repositories hosted in under an Artifactory Docker repository.
+
+=cut
+
+sub list_docker_repositories {
+    my $self       = shift;
+    my $repository = $self->repository();
+    my $url        = $self->_api_url() . "/docker/$repository/v2/_catalog";
+    return $self->get($url);
+}
+
 =head1 SECURITY
 
 =cut
@@ -1212,6 +1323,243 @@ sub delete_user {
     return $self->_handle_security( $user, 'delete', 'users' );
 }
 
+=head2 expire_password_for_a_single_user( $user )
+
+Expires a user's password
+
+=cut
+
+sub expire_password_for_a_single_user {
+    my ( $self, $user ) = @_;
+    my $url = $self->_api_url() . "/security/users/authorization/expirePassword/$user";
+    return $self->post($url);
+}
+
+=head2 expire_password_for_multiple_users( $user1, $user2 )
+
+Expires password for a list of users
+
+=cut
+
+sub expire_password_for_multiple_users {
+    my ( $self, @users ) = @_;
+    my $url = $self->_api_url() . "/security/users/authorization/expirePassword";
+    return $self->post(
+        $url,
+        'Content-Type' => 'application/json',
+        content        => $self->_json->encode( [@users] )
+    );
+}
+
+=head2 expire_password_for_all_users
+
+Expires password for all users
+
+=cut
+
+sub expire_password_for_all_users {
+    my ( $self, @users ) = @_;
+    my $url = $self->_api_url() . "/security/users/authorization/expirePasswordForAllUsers";
+    return $self->post($url);
+}
+
+=head2 unexpire_password_for_a_single_user( $user )
+
+Unexpires a user's password
+
+=cut
+
+sub unexpire_password_for_a_single_user {
+    my ( $self, $user ) = @_;
+    my $url = $self->_api_url() . "/security/users/authorization/unexpirePassword/$user";
+    return $self->post($url);
+}
+
+=head2 change_password( user => 'david', oldPassword => 'foo', newPassword => 'bar' )
+
+Changes a user's password
+
+=cut
+
+sub change_password {
+    my ( $self, %info ) = @_;
+    my $url         = $self->_api_url() . "/security/users/authorization/changePassword";
+    my $newpassword = delete $info{newPassword};
+    $info{newPassword1} = $newpassword;
+    $info{newPassword2} = $newpassword;    # API requires new passwords twice, once for verification
+
+    return $self->post(
+        $url,
+        'Content-Type' => 'application/json',
+        content        => $self->_json->encode( \%info )
+    );
+}
+
+=head2 get_password_expiration_policy
+
+Retrieves the password expiration policy
+
+=cut
+
+sub get_password_expiration_policy {
+    my $self = shift;
+    my $url  = $self->_api_url() . "/security/configuration/passwordExpirationPolicy";
+    return $self->get($url);
+}
+
+=head2 set_password_expiration_policy
+
+Sets the password expiration policy
+
+=cut
+
+sub set_password_expiration_policy {
+    my ( $self, %info ) = @_;
+    my $url = $self->_api_url() . "/security/configuration/passwordExpirationPolicy";
+    return $self->put(
+        $url,
+        'Content-Type' => 'application/json',
+        content        => $self->_json->encode( \%info )
+    );
+}
+
+=head2 configure_user_lock_policy( enabled => 'true|false', loginAttempts => $num )
+
+Configures the user lock policy that locks users out of their account if the number of repeated incorrect login attempts
+exceeds the configured maximum allowed.
+
+=cut
+
+sub configure_user_lock_policy {
+    my ( $self, %info ) = @_;
+    my $url = $self->_api_url() . "/security/userLockPolicy";
+    return $self->put(
+        $url,
+        'Content-Type' => 'application/json',
+        content        => $self->_json->encode( \%info )
+    );
+}
+
+=head2 retrieve_user_lock_policy
+
+Retrieves the currently configured user lock policy.
+
+=cut
+
+sub retrieve_user_lock_policy {
+    my $self = shift;
+    my $url  = $self->_api_url() . "/security/userLockPolicy";
+    return $self->get($url);
+}
+
+=head2 get_locked_out_users
+
+If locking out users is enabled, lists all users that were locked out due to recurrent incorrect login attempts.
+
+=cut
+
+sub get_locked_out_users {
+    my $self = shift;
+    my $url  = $self->_api_url() . "/security/lockedUsers";
+    return $self->get($url);
+}
+
+=head2 unlock_locked_out_user
+
+Unlocks a single user that was locked out due to recurrent incorrect login attempts.
+
+=cut
+
+sub unlock_locked_out_user {
+    my ( $self, $name ) = @_;
+    my $url = $self->_api_url() . "/security/unlockUsers/$name";
+    return $self->post($url);
+}
+
+=head2 unlock_locked_out_users
+
+Unlocks a list of users that were locked out due to recurrent incorrect login attempts.
+
+=cut
+
+sub unlock_locked_out_users {
+    my ( $self, @users ) = @_;
+    my $url = $self->_api_url() . "/security/unlockUsers";
+    return $self->post(
+        $url,
+        'Content-Type' => 'application/json',
+        content        => $self->_json->encode( \@users )
+    );
+}
+
+=head2 unlock_all_locked_out_users
+
+Unlocks all users that were locked out due to recurrent incorrect login attempts.
+
+=cut
+
+sub unlock_all_locked_out_users {
+    my $self = shift;
+    my $url  = $self->_api_url() . "/security/unlockAllUsers";
+    return $self->post($url);
+}
+
+=head2 create_api_key( apiKey => '3OloposOtVFyCMrT+cXmCAScmVMPrSYXkWIjiyDCXsY=' )
+
+Create an API key for the current user
+
+=cut
+
+sub create_api_key {
+    my ( $self, %args ) = @_;
+    return $self->_handle_api_key( 'post', %args );
+}
+
+=head2 get_api_key
+
+Get the current user's own API key
+
+=cut
+
+sub get_api_key {
+    my $self = shift;
+    return $self->_handle_api_key('get');
+}
+
+=head2 revoke_api_key
+
+Revokes the current user's API key
+
+=cut
+
+sub revoke_api_key {
+    my $self = shift;
+    return $self->_handle_revoke_api_key('/apiKey/auth');
+}
+
+=head2 revoke_user_api_key
+
+Revokes the API key of another user
+
+=cut
+
+sub revoke_user_api_key {
+    my ( $self, $user ) = @_;
+    return $self->_handle_revoke_api_key("/apiKey/auth/$user");
+}
+
+=head2 revoke_all_api_keys
+
+Revokes all API keys currently defined in the system
+
+=cut
+
+sub revoke_all_api_keys {
+    my ( $self, %args ) = @_;
+    my $deleteall = ( defined $args{deleteAll} ) ? $args{deleteAll} : 1;
+    return $self->_handle_revoke_api_key("/apiKey?deleteAll=$deleteall");
+}
+
 =head2 get_groups
 
 Get the groups list
@@ -1559,6 +1907,31 @@ sub calculate_debian_repository_metadata {
     return $self->_handle_repository_reindex( "/deb/reindex/$repository", %args );
 }
 
+=head2 calculate_opkg_repository_metadata( async => 0/1, writeProps => 1 )
+
+Calculates/recalculates the Packages and Release metadata for this repository,based on the ipk packages in it (in each
+feed location).
+
+=cut
+
+sub calculate_opkg_repository_metadata {
+    my ( $self, %args ) = @_;
+    my $repository = $self->repository();
+    return $self->_handle_repository_reindex( "/opkg/reindex/$repository", %args );
+}
+
+=head2 calculate_bower_index
+
+Recalculates the index for a Bower repository.
+
+=cut
+
+sub calculate_bower_index {
+    my $self       = shift;
+    my $repository = $self->repository();
+    return $self->_handle_repository_reindex("/bower/$repository/reindex");
+}
+
 =head1 SYSTEM & CONFIGURATION
 
 =cut
@@ -1673,6 +2046,49 @@ sub version_and_addons_information {
     return $self->get($url);
 }
 
+=head2 get_reverse_proxy_configuration
+
+Retrieves the reverse proxy configuration
+
+=cut
+
+sub get_reverse_proxy_configuration {
+    my $self = shift;
+
+    my $url = $self->_api_url() . "/system/configuration/webServer";
+    return $self->get($url);
+}
+
+=head2 update_reverse_proxy_configuration(%data)
+
+Updates the reverse proxy configuration
+
+=cut
+
+sub update_reverse_proxy_configuration {
+    my ( $self, %data ) = @_;
+
+    my $url = $self->_api_url() . "/system/configuration/webServer";
+    return $self->post(
+        $url,
+        'Content-Type' => 'application/json',
+        content        => $self->_json->encode( \%data )
+    );
+}
+
+=head2 get_reverse_proxy_snippet
+
+Gets the reverse proxy configuration snippet in text format
+
+=cut
+
+sub get_reverse_proxy_snippet {
+    my $self = shift;
+
+    my $url = $self->_api_url() . "/system/configuration/reverseProxy/nginx";
+    return $self->get($url);
+}
+
 =head1 PLUGINS
 
 =cut
@@ -1827,6 +2243,60 @@ sub export_system {
     return $self->_handle_system_settings( 'export', %args );
 }
 
+=head2 create_bundle( %hash of data structure )
+
+Create a new support bundle
+
+=cut
+
+sub create_bundle {
+    my ( $self, %args ) = @_;
+    my $url = $self->_api_url() . '/support/bundles';
+    %args = () unless %args;
+
+    return $self->post(
+        $url,
+        "Content-Type" => 'application/json',
+        Content        => $self->_json->encode( \%args )
+    );
+}
+
+=head2 list_bundles
+
+Lists previously created bundle currently stored in the system
+
+=cut
+
+sub list_bundles {
+    my $self = shift;
+    my $url  = $self->_api_url() . '/support/bundles';
+    return $self->get( $url, "Content-Type" => 'application/json', );
+}
+
+=head2 get_bundle( $name )
+
+Downloads a previously created bundle currently stored in the system
+
+=cut
+
+sub get_bundle {
+    my ( $self, $bundle ) = @_;
+    my $url = $self->_api_url() . '/support/bundles/' . $bundle;
+    return $self->get( $url, "Content-Type" => 'application/json', );
+}
+
+=head2 delete_bundle( $name )
+
+Deletes a previously created bundle from the system.
+
+=cut
+
+sub delete_bundle {
+    my ( $self, $bundle ) = @_;
+    my $url = $self->_api_url() . '/support/bundles/' . $bundle;
+    return $self->delete( $url, "Content-Type" => 'application/json', );
+}
+
 sub _build_ua {
     my $self = shift;
     return LWP::UserAgent->new( agent => 'perl-artifactory-client/' . $VERSION, );
@@ -2106,6 +2576,28 @@ sub _gather_delete_builds_params {
     return @params;
 }
 
+sub _handle_api_key {
+    my ( $self, $method, %args ) = @_;
+
+    my $url = $self->_api_url() . "/apiKey/auth";
+    return $self->$method(
+        $url,
+        'Content-Type' => 'application/json',
+        content        => $self->_json->encode( \%args )
+    );
+}
+
+sub _handle_revoke_api_key {
+    my ( $self, $endpoint ) = @_;
+
+    my $resp    = $self->get_api_key();
+    my $content = $self->_json->decode( $resp->content );
+    my %header;
+    $header{'X-Api-Key'} = $content->{apiKey};
+    my $url = $self->_api_url() . $endpoint;
+    return $self->delete( $url, %header );
+}
+
 __PACKAGE__->meta->make_immutable;
 
 =head1 AUTHOR
